from edge import eventlog
from edge.types import JnodeError
from ..types import LandmarkServiceArgs


class JsonLandmarkServiceArgs(LandmarkServiceArgs):
    def __init__(self, *args, **kwargs):
        LandmarkServiceArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(conf_jnode):
        """
        :type conf_jnode: dict
        :rtype: JsonLandmarkServiceArgs
        :raises JnodeError:
        """
        camera_jnode = conf_jnode.get('args', {}).get('camera')
        if camera_jnode is None:
            eventlog.log_e_param('#/args/camera')
            raise JnodeError

        # deserialize position
        position_jnode = camera_jnode.get('position')
        if position_jnode is None:
            eventlog.log_e_param('#/position')
            raise JnodeError

        forward = position_jnode.get('forward')
        left = position_jnode.get('left')
        up = position_jnode.get('up')

        if None in [forward, left, up]:
            eventlog.log_e_param('#/position')
            raise JnodeError

        # deserialize rotation
        rotation_jnode = camera_jnode.get('rotation')
        if rotation_jnode is None:
            eventlog.log_e_param('#/rotation')
            raise JnodeError

        elevation = rotation_jnode.get('elevation')
        azimuth = rotation_jnode.get('azimuth')
        pitch = rotation_jnode.get('pitch')

        if None in [elevation, azimuth, pitch]:
            eventlog.log_e_param('#/rotation')
            raise JnodeError

        # deserialize frustrum
        frustum_jnode = camera_jnode.get('frustum')

        if frustum_jnode is None:
            eventlog.log_e_param('#/frustum')
            raise JnodeError

        focal_dist = frustum_jnode.get('focal-dist')
        if focal_dist is None:
            eventlog.log_e_param('#/frustum/focal-dist')
            raise JnodeError

        sensor_jnode = frustum_jnode.get('sensor-wh')
        if sensor_jnode is None or len(sensor_jnode) != 2:
            eventlog.log_e_param('#/frustrum/sensor-wh')
            raise JnodeError

        sensor_w_mm, sensor_h_mm = sensor_jnode

        clip_jnode = frustum_jnode.get('forward')
        if clip_jnode is None or len(clip_jnode) != 2:
            eventlog.log_e_param('#/frustrum/forward')
            raise JnodeError

        clip_near, clip_far = clip_jnode

        # deserialize image-wh
        image_jnode = camera_jnode.get('image-wh')

        if image_jnode is None or len(image_jnode) != 2:
            eventlog.log_e_param('#/image-wh')
            raise JnodeError

        sensor_w_px, sensor_h_px = image_jnode

        return JsonLandmarkServiceArgs(
            left, up, forward, elevation, azimuth, pitch, clip_near, clip_far,
            sensor_w_mm, sensor_h_mm, sensor_w_px, sensor_h_px, focal_dist)
